@@ -89,7 +89,6 @@ where
     // Initialisations
     let im2d_arr: ArrayView3<F> = im2d.into();
     let kernel_weights_arr: ArrayView4<F> = kernel_weights.into();
-    let output: Array3<F>;
     let im2d_stride: Array3<F>;
     let new_im_height: usize;
     let new_im_width: usize;
@@ -118,28 +117,29 @@ where
     new_im_width = (im_width - 1) * stride + kernel_width;
 
     // weights.reshape(F, HH*WW*C)
-    // loop over each filter and flip the kernel and append it back
-    let mut filter_col: Array2<F> =
-        Array::zeros((num_channels_out, kernel_height * kernel_width * num_filters));
+    // loop over F and C ; flip the kernel and append it back
+    let mut filter_col: Array3<F> =
+        Array::zeros((num_channels_out, num_filters, kernel_height * kernel_width));
     for i in 0..num_channels_out {
-        let patch_kernel_weights = kernel_weights_arr.slice(s![i, .., .., ..]);
-        let mut weights_flatten = patch_kernel_weights
-            .into_shape(kernel_height * kernel_width * num_filters)
-            .unwrap()
-            .to_vec();
-        //FLIP
-        weights_flatten.reverse();
-        let weights_reverse =
-            Array::from_shape_vec(kernel_height * kernel_width * num_filters, weights_flatten)
-                .unwrap();
-        filter_col
-            .slice_mut(s![i, 0..kernel_height * kernel_width * num_filters])
-            .assign(&weights_reverse);
+        for j in 0..num_filters {
+            let patch_kernel_weights = kernel_weights_arr.slice(s![i, j, .., ..]);
+            let mut weights_flatten = patch_kernel_weights
+                .into_shape(kernel_height * kernel_width)
+                .unwrap()
+                .to_vec();
+            //FLIP
+            weights_flatten.reverse();
+            let weights_reverse =
+                Array::from_shape_vec(kernel_height * kernel_width, weights_flatten).unwrap();
+            filter_col
+                .slice_mut(s![i, j, 0..kernel_height * kernel_width])
+                .assign(&weights_reverse);
+        }
     }
-
-    let filter_col_flatten = filter_col
-        .into_shape((num_filters, kernel_height * kernel_width * num_channels_out))
-        .unwrap();
+    filter_col.swap_axes(0, 1);
+    let target_shape = (num_filters, num_channels_out * kernel_height * kernel_width);
+    let filter_col_flatten =
+        Array::from_shape_vec(target_shape, filter_col.iter().map(|a| *a).collect()).unwrap();
 
     // STRIDE > 1
     if stride != 1 {
@@ -198,10 +198,12 @@ where
     let mul = im_col.dot(&filter_transpose); // + bias_m
 
     if padding == Padding::Same {
-        let mul_reshape = mul
-            .into_shape((num_filters, new_im_height, new_im_width))
+        let mut mul_reshape = mul
+            .into_shape((new_im_height, new_im_width, num_filters))
             .unwrap()
             .into_owned();
+        mul_reshape.swap_axes(0, 2);
+        mul_reshape.swap_axes(1, 2);
         let (_, _, pad_top, pad_bottom, pad_left, pad_right) = get_padding_size(
             im_height * stride,
             im_width * stride,
@@ -212,14 +214,14 @@ where
 
         let pad_right_int = new_im_width - pad_right;
         let pad_bottom_int = new_im_height - pad_bottom;
-        output = mul_reshape
+        mul_reshape
             .slice(s![.., pad_top..pad_bottom_int, pad_left..pad_right_int])
-            .into_owned();
+            .into_owned()
     } else {
-        output = mul
+        let mul_transpose = mul.t();
+        mul_transpose
             .into_shape((num_filters, new_im_height, new_im_width))
             .unwrap()
-            .into_owned();
-    };
-    output
+            .into_owned()
+    }
 }
